package com.logicbig.example;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.Query;
import java.sql.Timestamp;
import java.time.LocalDate;
import java.time.ZoneOffset;
import java.util.List;

public class ExampleMain {
    private static EntityManagerFactory entityManagerFactory =
            Persistence.createEntityManagerFactory("example-unit");

    public static void main(String[] args) {
        try {
            persistEmployees();
            findEmployeeBySalaryRange();
            findEmployeeWithLessThanAverageSalary();
        } finally {
            entityManagerFactory.close();
        }
    }

    private static void findEmployeeBySalaryRange() {
        System.out.println("-- Employee by salary range --");
        EntityManager em = entityManagerFactory.createEntityManager();
        Query query = em.createNamedQuery(Employee.SALARY_RANGE_QUERY);
        query.setParameter("min", 2000L);
        query.setParameter("max", 4000L);
        List<Employee> resultList = query.getResultList();
        resultList.forEach(System.out::println);
        em.close();
    }

    private static void findEmployeeWithLessThanAverageSalary() {
        System.out.println("-- Employee with salary less than average --");
        EntityManager em = entityManagerFactory.createEntityManager();
        Query query = em.createNamedQuery(Employee.SALARY_LESS_THAN_AVERAGE_QUERY);
        List<Employee> resultList = query.getResultList();
        resultList.forEach(System.out::println);
        em.close();
    }

    public static void persistEmployees() {
        Employee employee1 = Employee.create("Diana", 3000, LocalDate.of(1999, 11, 15));
        Employee employee2 = Employee.create("Rose", 4000, LocalDate.of(2011, 5, 1));
        Employee employee3 = Employee.create("Denise", 1500, LocalDate.of(2006, 1, 10));
        Employee employee4 = Employee.create("Mike", 2000, LocalDate.of(2015, 8, 20));
        EntityManager em = entityManagerFactory.createEntityManager();
        em.getTransaction().begin();
        em.persist(employee1);
        em.persist(employee2);
        em.persist(employee3);
        em.persist(employee4);
        em.getTransaction().commit();
        em.close();
    }

    private static Timestamp localToTimeStamp(LocalDate date) {
        return Timestamp.from(date.atStartOfDay().toInstant(ZoneOffset.UTC));
    }
}